"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseOctree = parseOctree;
exports.parseMetadata = parseMetadata;
exports["default"] = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var THREE = _interopRequireWildcard(require("three"));

var _GeometryLayer2 = _interopRequireDefault(require("./GeometryLayer"));

var _Fetcher = _interopRequireDefault(require("../Provider/Fetcher"));

var _PotreeBinParser = _interopRequireDefault(require("../Parser/PotreeBinParser"));

var _PotreeCinParser = _interopRequireDefault(require("../Parser/PotreeCinParser"));

var _PointsMaterial = _interopRequireWildcard(require("../Renderer/PointsMaterial"));

var _Picking = _interopRequireDefault(require("../Core/Picking"));

var _Extent = _interopRequireDefault(require("../Core/Geographic/Extent"));

var _CancelledCommandException = _interopRequireDefault(require("../Core/Scheduler/CancelledCommandException"));

var point = new THREE.Vector3(); // Draw a cube with lines (12 lines).

function cube(size) {
  var h = size.clone().multiplyScalar(0.5);
  var geometry = new THREE.Geometry();
  var line = new THREE.Line(geometry);
  geometry.vertices.push(new THREE.Vector3(-h.x, -h.y, -h.z), new THREE.Vector3(-h.x, h.y, -h.z), new THREE.Vector3(-h.x, h.y, -h.z), new THREE.Vector3(h.x, h.y, -h.z), new THREE.Vector3(h.x, h.y, -h.z), new THREE.Vector3(h.x, -h.y, -h.z), new THREE.Vector3(h.x, -h.y, -h.z), new THREE.Vector3(-h.x, -h.y, -h.z), new THREE.Vector3(-h.x, -h.y, h.z), new THREE.Vector3(-h.x, h.y, h.z), new THREE.Vector3(-h.x, h.y, h.z), new THREE.Vector3(h.x, h.y, h.z), new THREE.Vector3(h.x, h.y, h.z), new THREE.Vector3(h.x, -h.y, h.z), new THREE.Vector3(h.x, -h.y, h.z), new THREE.Vector3(-h.x, -h.y, h.z), new THREE.Vector3(-h.x, -h.y, -h.z), new THREE.Vector3(-h.x, -h.y, h.z), new THREE.Vector3(-h.x, h.y, -h.z), new THREE.Vector3(-h.x, h.y, h.z), new THREE.Vector3(h.x, h.y, -h.z), new THREE.Vector3(h.x, h.y, h.z), new THREE.Vector3(h.x, -h.y, -h.z), new THREE.Vector3(h.x, -h.y, h.z));
  line.computeLineDistances();
  return line.geometry;
}

function initBoundingBox(elt, layer) {
  var size = new THREE.Vector3();
  elt.tightbbox.getSize(size);
  elt.obj.boxHelper = new THREE.LineSegments(cube(size), elt.childrenBitField ? new THREE.LineDashedMaterial({
    color: 0,
    dashSize: 0.25,
    gapSize: 0.25
  }) : new THREE.LineBasicMaterial({
    color: 0
  }));
  elt.obj.boxHelper.frustumCulled = false;
  elt.obj.boxHelper.position.copy(elt.tightbbox.min);
  elt.obj.boxHelper.position.add(size.multiplyScalar(0.5));
  elt.obj.boxHelper.updateMatrixWorld(true);
  elt.obj.boxHelper.autoUpdateMatrix = false;
  elt.obj.boxHelper.material.linewidth = 2;
  elt.obj.boxHelper.layers.mask = layer.bboxes.layers.mask;
  layer.bboxes.add(elt.obj.boxHelper);
  elt.obj.boxHelper.updateMatrixWorld();
}

function computeScreenSpaceError(context, layer, elt, distance) {
  if (distance <= 0) {
    return Infinity;
  }

  var pointSpacing = layer.metadata.spacing / Math.pow(2, elt.name.length); // Estimate the onscreen distance between 2 points

  var onScreenSpacing = context.camera.preSSE * pointSpacing / distance; // [  P1  ]--------------[   P2   ]
  //     <--------------------->      = pointsSpacing (in world coordinates)
  //                                  ~ onScreenSpacing (in pixels)
  // <------>                         = layer.pointSize (in pixels)

  return Math.max(0.0, onScreenSpacing - layer.pointSize);
}

function markForDeletion(elt) {
  if (elt.obj) {
    elt.obj.material.visible = false;
  }

  if (!elt.notVisibleSince) {
    elt.notVisibleSince = Date.now(); // Set .sse to an invalid value

    elt.sse = -1;
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = elt.children[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var child = _step.value;
      markForDeletion(child);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
} // Create an A(xis)A(ligned)B(ounding)B(ox) for the child `childIndex` of one aabb.
// (PotreeConverter protocol builds implicit octree hierarchy by applying the same
// subdivision algo recursively)


var dHalfLength = new THREE.Vector3();

function createChildAABB(aabb, childIndex) {
  // Code inspired from potree
  var box = aabb.clone();
  aabb.getCenter(box.max);
  dHalfLength.copy(box.max).sub(aabb.min);

  if (childIndex === 1) {
    box.min.z += dHalfLength.z;
    box.max.z += dHalfLength.z;
  } else if (childIndex === 3) {
    box.min.z += dHalfLength.z;
    box.max.z += dHalfLength.z;
    box.min.y += dHalfLength.y;
    box.max.y += dHalfLength.y;
  } else if (childIndex === 0) {//
  } else if (childIndex === 2) {
    box.min.y += dHalfLength.y;
    box.max.y += dHalfLength.y;
  } else if (childIndex === 5) {
    box.min.z += dHalfLength.z;
    box.max.z += dHalfLength.z;
    box.min.x += dHalfLength.x;
    box.max.x += dHalfLength.x;
  } else if (childIndex === 7) {
    box.min.add(dHalfLength);
    box.max.add(dHalfLength);
  } else if (childIndex === 4) {
    box.min.x += dHalfLength.x;
    box.max.x += dHalfLength.x;
  } else if (childIndex === 6) {
    box.min.y += dHalfLength.y;
    box.max.y += dHalfLength.y;
    box.min.x += dHalfLength.x;
    box.max.x += dHalfLength.x;
  }

  return box;
}

function parseOctree(layer, hierarchyStepSize, root) {
  return _Fetcher["default"].arrayBuffer("".concat(root.baseurl, "/r").concat(root.name, ".hrc"), layer.fetchOptions).then(function (blob) {
    var view = new DataView(blob);
    var stack = [];
    var offset = 0;
    root.childrenBitField = view.getUint8(0);
    offset += 1;
    root.numPoints = view.getUint32(1, true);
    offset += 4;
    root.children = [];
    stack.push(root);

    while (stack.length && offset < blob.byteLength) {
      var snode = stack.shift(); // look up 8 children

      for (var i = 0; i < 8; i++) {
        // does snode have a #i child ?
        if (snode.childrenBitField & 1 << i && offset + 5 <= blob.byteLength) {
          var c = view.getUint8(offset);
          offset += 1;
          var n = view.getUint32(offset, true);
          offset += 4;

          if (n == 0) {
            n = root.numPoints;
          }

          var childname = snode.name + i;
          var bounds = createChildAABB(snode.bbox, i);
          var url = root.baseurl;

          if (childname.length % hierarchyStepSize == 0) {
            var myname = childname.substr(root.name.length);
            url = "".concat(root.baseurl, "/").concat(myname);
          }

          var item = {
            numPoints: n,
            childrenBitField: c,
            children: [],
            name: childname,
            baseurl: url,
            bbox: bounds,
            layer: layer,
            parent: snode
          };
          snode.children.push(item);
          stack.push(item);
        }
      }
    }

    return root;
  });
}

function findChildrenByName(node, name) {
  if (node.name === name) {
    return node;
  }

  var charIndex = node.name.length;

  for (var i = 0; i < node.children.length; i++) {
    if (node.children[i].name[charIndex] == name[charIndex]) {
      return findChildrenByName(node.children[i], name);
    }
  }

  throw new Error("Cannot find node with name '".concat(name, "'"));
}

function computeBbox(layer) {
  var bbox;

  if (layer.isFromPotreeConverter) {
    bbox = new THREE.Box3(new THREE.Vector3(layer.metadata.boundingBox.lx, layer.metadata.boundingBox.ly, layer.metadata.boundingBox.lz), new THREE.Vector3(layer.metadata.boundingBox.ux, layer.metadata.boundingBox.uy, layer.metadata.boundingBox.uz));
  } else {
    // lopocs
    var idx = 0;
    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = layer.metadata[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        var entry = _step2.value;

        if (entry.table == layer.table) {
          break;
        }

        idx++;
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
          _iterator2["return"]();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }

    bbox = new THREE.Box3(new THREE.Vector3(layer.metadata[idx].bbox.xmin, layer.metadata[idx].bbox.ymin, layer.metadata[idx].bbox.zmin), new THREE.Vector3(layer.metadata[idx].bbox.xmax, layer.metadata[idx].bbox.ymax, layer.metadata[idx].bbox.zmax));
  }

  return bbox;
}

function parseMetadata(metadata, layer) {
  layer.metadata = metadata;
  var customBinFormat = true; // Lopocs pointcloud server can expose the same file structure as PotreeConverter output.
  // The only difference is the metadata root file (cloud.js vs infos/sources), and we can
  // check for the existence of a `scale` field.
  // (if `scale` is defined => we're fetching files from PotreeConverter)

  if (layer.metadata.scale != undefined) {
    layer.isFromPotreeConverter = true; // PotreeConverter format

    customBinFormat = layer.metadata.pointAttributes === 'CIN'; // do we have normal information

    var normal = Array.isArray(layer.metadata.pointAttributes) && layer.metadata.pointAttributes.find(function (elem) {
      return elem.startsWith('NORMAL');
    });

    if (normal) {
      layer.material.defines[normal] = 1;
    }
  } else {
    // Lopocs
    layer.metadata.scale = 1;
    layer.metadata.octreeDir = "itowns/".concat(layer.table, ".points");
    layer.metadata.hierarchyStepSize = 1000000; // ignore this with lopocs

    customBinFormat = true;
  }

  layer.parse = customBinFormat ? _PotreeCinParser["default"].parse : _PotreeBinParser["default"].parse;
  layer.extension = customBinFormat ? 'cin' : 'bin';
  layer.supportsProgressiveDisplay = customBinFormat;
}

var PointCloudLayer =
/*#__PURE__*/
function (_GeometryLayer) {
  (0, _inherits2["default"])(PointCloudLayer, _GeometryLayer);

  /**
   * Constructs a new instance of point cloud layer.
   * @constructor
   * @extends GeometryLayer
   *
   * @example
   * // Create a new point cloud layer
   * const points = new PointCloudLayer('points',
   *  {
   *      url: 'https://pointsClouds/',
   *      file: 'points.js',
   *  }, view);
   *
   * View.prototype.addLayer.call(view, points);
   *
   * @param      {string}  id - The id of the layer, that should be unique. It is
   * not mandatory, but an error will be emitted if this layer is added a
   * {@link View} that already has a layer going by that id.
   * @param      {object}  config   configuration, all elements in it
   * will be merged as is in the layer.
   * @param {string} config.url The base url.
   * @param {string} config.file file name.
   * @param {number} [config.pointBudget=2000000] max displayed points count.
   * @param {number} [config.sseThreshold=2] screen space error Threshold.
   * @param {Object} [config.fetchOptions={}] fetch config.
   * @param {number} [config.pointSize=4] point size.
   * @param {THREE.Material} [config.material] override material.
   * @param {number} [config.mode=MODE.COLOR] displaying mode.
   *
   * @param  {View}  view  The view
   */
  function PointCloudLayer(id, config, view) {
    var _this;

    (0, _classCallCheck2["default"])(this, PointCloudLayer);

    if (!config.url) {
      throw new Error('New PointCloudLayer: url is required');
    }

    if (!config.file) {
      throw new Error('New PointCloudLayer: file is required');
    }

    _this = (0, _possibleConstructorReturn2["default"])(this, (0, _getPrototypeOf2["default"])(PointCloudLayer).call(this, id, new THREE.Group()));
    _this.isPointCloudLayer = true;
    _this.protocol = 'potreeconverter';
    _this.file = config.file;
    _this.url = config.url;
    _this.group = config.group || new THREE.Group();

    _this.object3d.add(_this.group);

    _this.bboxes = config.bboxes || new THREE.Group();
    _this.bboxes.visible = false;

    _this.object3d.add(_this.bboxes);

    _this.group.updateMatrixWorld(); // default config


    _this.fetchOptions = config.fetchOptions || {};
    _this.octreeDepthLimit = config.octreeDepthLimit || -1;
    _this.pointBudget = config.pointBudget || 2000000;
    _this.pointSize = config.pointSize === 0 || !isNaN(config.pointSize) ? config.pointSize : 4;
    _this.sseThreshold = config.sseThreshold || 2;
    _this.material = config.material || {};
    _this.material = _this.material.isMaterial ? config.material : new _PointsMaterial["default"](config.material);
    _this.material.defines = _this.material.defines || {};
    _this.mode = _PointsMaterial.MODE.COLOR || config.mode;
    _this.whenReady = _Fetcher["default"].json("".concat(_this.url, "/").concat(_this.file), _this.fetchOptions).then(function (metadata) {
      parseMetadata(metadata, (0, _assertThisInitialized2["default"])(_this));
      var bbox = computeBbox((0, _assertThisInitialized2["default"])(_this));
      return parseOctree((0, _assertThisInitialized2["default"])(_this), _this.metadata.hierarchyStepSize, {
        baseurl: "".concat(_this.url, "/").concat(_this.metadata.octreeDir, "/r"),
        name: '',
        bbox: bbox
      });
    }).then(function (root) {
      _this.root = root;
      root.findChildrenByName = findChildrenByName.bind(root, root);
      _this.extent = _Extent["default"].fromBox3(view.referenceCrs, root.bbox);
      return (0, _assertThisInitialized2["default"])(_this);
    });
    return _this;
  }

  (0, _createClass2["default"])(PointCloudLayer, [{
    key: "preUpdate",
    value: function preUpdate(context, changeSources) {
      // Bail-out if not ready
      if (!this.root) {
        return [];
      } // See https://cesiumjs.org/hosted-apps/massiveworlds/downloads/Ring/WorldScaleTerrainRendering.pptx
      // slide 17


      context.camera.preSSE = context.camera.height / (2 * Math.tan(THREE.Math.degToRad(context.camera.camera3D.fov) * 0.5));

      if (this.material) {
        this.material.visible = this.visible;
        this.material.opacity = this.opacity;
        this.material.transparent = this.opacity < 1;
        this.material.size = this.pointSize;
      } // lookup lowest common ancestor of changeSources


      var commonAncestorName;
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = changeSources.values()[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var source = _step3.value;

          if (source.isCamera || source == this) {
            // if the change is caused by a camera move, no need to bother
            // to find common ancestor: we need to update the whole tree:
            // some invisible tiles may now be visible
            return [this.root];
          }

          if (source.obj === undefined) {
            continue;
          } // filter sources that belong to our layer


          if (source.obj.isPoints && source.obj.layer == this) {
            if (!commonAncestorName) {
              commonAncestorName = source.name;
            } else {
              var i = void 0;

              for (i = 0; i < Math.min(source.name.length, commonAncestorName.length); i++) {
                if (source.name[i] != commonAncestorName[i]) {
                  break;
                }
              }

              commonAncestorName = commonAncestorName.substr(0, i);

              if (commonAncestorName.length == 0) {
                break;
              }
            }
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3["return"] != null) {
            _iterator3["return"]();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      if (commonAncestorName) {
        return [this.root.findChildrenByName(commonAncestorName)];
      } // Start updating from hierarchy root


      return [this.root];
    }
  }, {
    key: "update",
    value: function update(context, layer, elt) {
      var _this2 = this;

      elt.visible = false;

      if (this.octreeDepthLimit >= 0 && this.octreeDepthLimit < elt.name.length) {
        markForDeletion(elt);
        return;
      } // pick the best bounding box


      var bbox = elt.tightbbox ? elt.tightbbox : elt.bbox;
      elt.visible = context.camera.isBox3Visible(bbox, this.object3d.matrixWorld);

      if (!elt.visible) {
        markForDeletion(elt);
        return;
      }

      elt.notVisibleSince = undefined;
      point.copy(context.camera.camera3D.position).sub(this.object3d.position); // only load geometry if this elements has points

      if (elt.numPoints > 0) {
        if (elt.obj) {
          if (elt.obj.material.update) {
            elt.obj.material.update(this.material);
          } else {
            elt.obj.material.copy(this.material);
          }
        } else if (!elt.promise) {
          var distance = Math.max(0.001, bbox.distanceToPoint(point)); // Increase priority of nearest node

          var priority = computeScreenSpaceError(context, layer, elt, distance) / distance;
          elt.promise = context.scheduler.execute({
            layer: layer,
            requester: elt,
            view: context.view,
            priority: priority,
            redraw: true,
            isLeaf: elt.childrenBitField == 0,
            earlyDropFunction: function earlyDropFunction(cmd) {
              return !cmd.requester.visible || !_this2.visible;
            }
          }).then(function (pts) {
            if (_this2.onPointsCreated) {
              _this2.onPointsCreated(layer, pts);
            }

            elt.obj = pts; // store tightbbox to avoid ping-pong (bbox = larger => visible, tight => invisible)

            elt.tightbbox = pts.tightbbox; // make sure to add it here, otherwise it might never
            // be added nor cleaned

            _this2.group.add(elt.obj);

            elt.obj.updateMatrixWorld(true);
            elt.promise = null;
          }, function (err) {
            if (err instanceof _CancelledCommandException["default"]) {
              elt.promise = null;
            }
          });
        }
      }

      if (elt.children && elt.children.length) {
        var _distance = bbox.distanceToPoint(point);

        elt.sse = computeScreenSpaceError(context, layer, elt, _distance) / this.sseThreshold;

        if (elt.sse >= 1) {
          return elt.children;
        } else {
          var _iteratorNormalCompletion4 = true;
          var _didIteratorError4 = false;
          var _iteratorError4 = undefined;

          try {
            for (var _iterator4 = elt.children[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
              var child = _step4.value;
              markForDeletion(child);
            }
          } catch (err) {
            _didIteratorError4 = true;
            _iteratorError4 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion4 && _iterator4["return"] != null) {
                _iterator4["return"]();
              }
            } finally {
              if (_didIteratorError4) {
                throw _iteratorError4;
              }
            }
          }
        }
      }
    }
  }, {
    key: "postUpdate",
    value: function postUpdate() {
      if (!this.group) {
        return;
      }

      this.displayedCount = 0;
      var _iteratorNormalCompletion5 = true;
      var _didIteratorError5 = false;
      var _iteratorError5 = undefined;

      try {
        for (var _iterator5 = this.group.children[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
          var _pts2 = _step5.value;

          if (_pts2.material.visible) {
            var _count2 = _pts2.geometry.attributes.position.count;

            _pts2.geometry.setDrawRange(0, _count2);

            this.displayedCount += _count2;
          }
        }
      } catch (err) {
        _didIteratorError5 = true;
        _iteratorError5 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion5 && _iterator5["return"] != null) {
            _iterator5["return"]();
          }
        } finally {
          if (_didIteratorError5) {
            throw _iteratorError5;
          }
        }
      }

      if (this.displayedCount > this.pointBudget) {
        // 2 different point count limit implementation, depending on the pointcloud source
        if (this.supportsProgressiveDisplay) {
          // In this format, points are evenly distributed within a node,
          // so we can draw a percentage of each node and still get a correct
          // representation
          var reduction = this.pointBudget / this.displayedCount;
          var _iteratorNormalCompletion6 = true;
          var _didIteratorError6 = false;
          var _iteratorError6 = undefined;

          try {
            for (var _iterator6 = this.group.children[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
              var pts = _step6.value;

              if (pts.material.visible) {
                var count = Math.floor(pts.geometry.drawRange.count * reduction);

                if (count > 0) {
                  pts.geometry.setDrawRange(0, count);
                } else {
                  pts.material.visible = false;
                }
              }
            }
          } catch (err) {
            _didIteratorError6 = true;
            _iteratorError6 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion6 && _iterator6["return"] != null) {
                _iterator6["return"]();
              }
            } finally {
              if (_didIteratorError6) {
                throw _iteratorError6;
              }
            }
          }

          this.displayedCount *= reduction;
        } else {
          // This format doesn't require points to be evenly distributed, so
          // we're going to sort the nodes by "importance" (= on screen size)
          // and display only the first N nodes
          this.group.children.sort(function (p1, p2) {
            return p2.userData.metadata.sse - p1.userData.metadata.sse;
          });
          var limitHit = false;
          this.displayedCount = 0;
          var _iteratorNormalCompletion7 = true;
          var _didIteratorError7 = false;
          var _iteratorError7 = undefined;

          try {
            for (var _iterator7 = this.group.children[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
              var _pts = _step7.value;
              var _count = _pts.geometry.attributes.position.count;

              if (limitHit || this.displayedCount + _count > this.pointBudget) {
                _pts.material.visible = false;
                limitHit = true;
              } else {
                this.displayedCount += _count;
              }
            }
          } catch (err) {
            _didIteratorError7 = true;
            _iteratorError7 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion7 && _iterator7["return"] != null) {
                _iterator7["return"]();
              }
            } finally {
              if (_didIteratorError7) {
                throw _iteratorError7;
              }
            }
          }
        }
      }

      var now = Date.now();

      for (var i = this.group.children.length - 1; i >= 0; i--) {
        var obj = this.group.children[i];

        if (!obj.material.visible && now - obj.userData.metadata.notVisibleSince > 10000) {
          // remove from group
          this.group.children.splice(i, 1);

          if (Array.isArray(obj.material)) {
            var _iteratorNormalCompletion8 = true;
            var _didIteratorError8 = false;
            var _iteratorError8 = undefined;

            try {
              for (var _iterator8 = obj.material[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
                var material = _step8.value;
                material.dispose();
              }
            } catch (err) {
              _didIteratorError8 = true;
              _iteratorError8 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion8 && _iterator8["return"] != null) {
                  _iterator8["return"]();
                }
              } finally {
                if (_didIteratorError8) {
                  throw _iteratorError8;
                }
              }
            }
          } else {
            obj.material.dispose();
          }

          obj.geometry.dispose();
          obj.material = null;
          obj.geometry = null;
          obj.userData.metadata.obj = null;
        }
      }
    }
  }, {
    key: "pickObjectsAt",
    value: function pickObjectsAt(view, mouse, radius) {
      return _Picking["default"].pickPointsAt(view, mouse, radius, this);
    }
  }], [{
    key: "getObjectToUpdateForAttachedLayers",
    value: function getObjectToUpdateForAttachedLayers(meta) {
      if (meta.obj) {
        var p = meta.parent;

        if (p && p.obj) {
          return {
            element: meta.obj,
            parent: p.obj
          };
        } else {
          return {
            element: meta.obj
          };
        }
      }
    }
  }]);
  return PointCloudLayer;
}(_GeometryLayer2["default"]);

var _default = PointCloudLayer;
exports["default"] = _default;