"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _TMSSource2 = _interopRequireDefault(require("./TMSSource"));

var _Fetcher = _interopRequireDefault(require("../Provider/Fetcher"));

function toTMSUrl(url) {
  return url.replace(/\{/g, '${');
}
/**
 * @classdesc
 * VectorTilesSource are object containing informations on how to fetch vector tiles resources.
 *
 * @property {string} style - the url to json style.
 * @property {string} sprite - the base url to sprites folder.
 * @property {function} filter - function to filter vector tiles layers, the parameter function is a layer.
 *
 */


var VectorTilesSource =
/*#__PURE__*/
function (_TMSSource) {
  (0, _inherits2["default"])(VectorTilesSource, _TMSSource);

  /**
   * @param {Object} source - An object that can contain all properties of a
   * VectorTilesSource and {@link Source}.
   *
   * @constructor
   */
  function VectorTilesSource(source) {
    var _this;

    (0, _classCallCheck2["default"])(this, VectorTilesSource);
    source.format = 'application/x-protobuf;type=mapbox-vector';
    source.projection = 'EPSG:3857';
    source.isInverted = true;
    source.url = source.url || '.';
    _this = (0, _possibleConstructorReturn2["default"])(this, (0, _getPrototypeOf2["default"])(VectorTilesSource).call(this, source));

    var ffilter = source.filter || function () {
      return true;
    };

    _this.filter = [];
    var promises = [];

    if (source.style) {
      promises.push(_Fetcher["default"].json(source.style).then(function (style) {
        var s = Object.keys(style.sources)[0];
        var os = style.sources[s];
        style.layers.forEach(function (layer) {
          if (layer.type === 'background') {
            _this.backgroundLayer = layer;
          } else if (ffilter(layer)) {
            _this.filter.push(layer);
          }
        });

        if (_this.url == '.') {
          if (os.url) {
            return _Fetcher["default"].json(os.url).then(function (tileJSON) {
              if (tileJSON.tiles[0]) {
                _this.url = toTMSUrl(tileJSON.tiles[0]);
              }
            });
          } else if (os.tiles[0]) {
            _this.url = toTMSUrl(os.tiles[0]);
          }
        }
      }));
    } else {
      throw new Error('New VectorTilesSource: style is required');
    }

    if (source.sprite) {
      promises.push(_Fetcher["default"].json("".concat(source.sprite, ".json")).then(function (sprites) {
        _this.sprites = sprites;
        return _Fetcher["default"].texture("".concat(source.sprite, ".png"), {
          crossOrigin: 'anonymous'
        }).then(function (texture) {
          _this.sprites.img = texture.image;
        });
      }));
    }

    _this.whenReady = Promise.all(promises);
    return _this;
  }

  return VectorTilesSource;
}(_TMSSource2["default"]);

var _default = VectorTilesSource;
exports["default"] = _default;